import unittest

import sbol3


class TestValidationReport(unittest.TestCase):

    def test_boolean(self):
        # False if no errors or warnings
        report = sbol3.ValidationReport()
        self.assertEqual(False, bool(report))
        # True if any errors
        report.addError(None, None, 'Fake error')
        self.assertEqual(True, bool(report))
        # True if any warnings
        report = sbol3.ValidationReport()
        report.addWarning(None, None, 'Fake warning')
        self.assertEqual(True, bool(report))
        # True if both errors and warnings
        report = sbol3.ValidationReport()
        report.addError(None, None, 'Fake error')
        report.addWarning(None, None, 'Fake warning')
        self.assertEqual(True, bool(report))

    def test_length(self):
        # Length should be the sum of errors and warnings
        report = sbol3.ValidationReport()
        self.assertEqual(0, len(report))
        report.addError(None, None, 'Fake error')
        self.assertEqual(1, len(report))
        report.addWarning(None, None, 'Fake warning')
        self.assertEqual(2, len(report))


if __name__ == '__main__':
    unittest.main()



import abc
import math
from typing import List

from . import *


class Feature(Identified, abc.ABC):
    """Feature is an abstract base class."""

    def __init__(self, identity: str, type_uri: str,
                 *, roles: List[str] = None, orientation: str = None,
                 name: str = None, description: str = None,
                 derived_from: List[str] = None,
                 generated_by: List[str] = None,
                 measures: List[SBOLObject] = None) -> None:
        super().__init__(identity=identity, type_uri=type_uri, name=name,
                         description=description, derived_from=derived_from,
                         generated_by=generated_by, measures=measures)
        self.roles = URIProperty(self, SBOL_ROLE, 0, math.inf,
                                 initial_value=roles)
        self.orientation = URIProperty(self, SBOL_ORIENTATION, 0, 1,
                                       initial_value=orientation)

    def validate(self, report: ValidationReport = None) -> ValidationReport:
        report = super().validate(report)
        # If there is an orientation, it must be in the valid set
        if self.orientation is not None:
            valid_orientations = [SBOL_INLINE, SBOL_REVERSE_COMPLEMENT]
            if self.orientation not in valid_orientations:
                message = f'{self.orientation} is not a valid orientation'
                report.addError(self.identity, None, message)
        return report



from typing import List

from . import *


class ExperimentalData(TopLevel):
    """The purpose of the ExperimentalData class is to aggregate links to
    experimental data files. An ExperimentalData is typically
    associated with a single sample, lab instrument, or experimental
    condition and can be used to describe the output of the test phase
    of a design-build-test-learn workflow.

    """

    def __init__(self, identity: str,
                 *, attachments: List[str] = None,
                 name: str = None, description: str = None,
                 derived_from: List[str] = None,
                 generated_by: List[str] = None,
                 measures: List[SBOLObject] = None,
                 type_uri: str = SBOL_EXPERIMENTAL_DATA):
        super().__init__(identity=identity, type_uri=type_uri,
                         attachments=attachments, name=name,
                         description=description, derived_from=derived_from,
                         generated_by=generated_by, measures=measures)


Document.register_builder(SBOL_EXPERIMENTAL_DATA, ExperimentalData)



import math
from typing import Union, List

from . import *
# Feature is not exported
from .feature import Feature


class SubComponent(Feature):
    """The SubComponent class can be used to specify structural
    hierarchy. For example, the Component of a gene might contain four
    SubComponent objects: a promoter, RBS, CDS, and terminator, each
    linked to a Component that provides the complete definition. In
    turn, the Component of the promoter SubComponent might itself
    contain SubComponent objects defining various operator sites, etc.

    """

    def __init__(self, instance_of: Union[Identified, str],
                 *, role_integration: str = None,
                 locations: List[Location] = None,
                 source_locations: List[Location] = None,
                 roles: List[str] = None, orientation: str = None,
                 name: str = None, description: str = None,
                 derived_from: List[str] = None,
                 generated_by: List[str] = None,
                 measures: List[SBOLObject] = None,
                 identity: str = None,
                 type_uri: str = SBOL_SUBCOMPONENT) -> None:
        super().__init__(identity=identity, type_uri=type_uri,
                         roles=roles, orientation=orientation, name=name,
                         description=description, derived_from=derived_from,
                         generated_by=generated_by, measures=measures)
        self.role_integration = URIProperty(self, SBOL_ROLE, 0, 1,
                                            initial_value=role_integration)
        self.instance_of = ReferencedObject(self, SBOL_INSTANCE_OF, 1, 1,
                                            initial_value=instance_of)
        self.source_locations = OwnedObject(self, SBOL_SOURCE_LOCATION, 0, math.inf,
                                            initial_value=source_locations,
                                            type_constraint=Location)
        self.locations = OwnedObject(self, SBOL_LOCATION, 0, math.inf,
                                     initial_value=locations,
                                     type_constraint=Location)


def build_subcomponent(identity: str, type_uri: str = SBOL_SUBCOMPONENT) -> Identified:
    """Used by Document to construct a SubComponent when reading an SBOL file.
    """
    missing = PYSBOL3_MISSING
    obj = SubComponent(missing, identity=identity, type_uri=type_uri)
    obj._properties[SBOL_INSTANCE_OF] = []
    return obj


Document.register_builder(SBOL_SUBCOMPONENT, build_subcomponent)
